"""Cryptographic helper functions."""


from blockchyp.crypto.auth import auth_headers
from blockchyp.crypto.certificate import TerminalAdapter
from blockchyp.crypto.symmetric import decrypt, encrypt



"""
Request and response field types.
"""

# pylint: disable=too-few-public-methods

class CardType:
    """Used to differentiate credit, debit, EBT, and gift cards."""

    CREDIT = 0
    DEBIT = 1
    EBT = 2
    GIFT = 3
    HEALTHCARE = 4


class SignatureFormat:
    """File formats for customer signatures."""

    NONE = "none"
    PNG = "png"
    JPG = "jpg"
    GIF = "gif"


class PromptType:
    """Specifies the type of text input for a terminal prompt."""

    AMOUNT = "amount"
    CUST_NUMBER = "customer-number"
    EMAIL = "email"
    PHONE = "phone"
    REWARDS_NUMBER = "rewards-number"
    FIRST_NAME = "first-name"
    LAST_NAME = "last-name"


class AVSResponse:
    """Contains address verification results."""

    NOT_APPLICABLE = ""
    NOT_SUPPORTED = "not_supported"
    RETRY = "retry"
    NO_MATCH = "no_match"
    ADDRESS_MATCH = "address_match"
    POSTAL_CODE_MATCH = "zip_match"
    ADDRESS_AND_POSTAL_CODE_MATCH = "match"

class CVMType:
    """Contains customer verification methods."""

    SIGNATURE = "Signature"
    OFFLINE_PIN = "Offline PIN"
    ONLINE_PIN = "Online PIN"
    CDCVM = "CDCVM"
    NO_CVM = "No CVM"

class HealthcareType:
    """Contains categories of healthcare amounts."""

    HEALTHCARE = "healthcare"
    PRESCRIPTION = "prescription"
    VISION = "vision"
    CLINIC = "clinic"
    DENTAL = "dental"

class RoundingMode:
    """Rounding mode options for fractional pennies."""

    UP = "up"
    NEAREST = "nearest"
    DOWN = "down"



import hashlib
import hmac
import os

from blockchyp.util import iso_timestamp


NONCE_SIZE = 32
HEADER_NONCE = "Nonce"
HEADER_TIMESTAMP = "Timestamp"
HEADER_AUTHORIZATION = "Dual"


def auth_headers(api_key, bearer_token, signing_key):
    # type: (str, str, str) -> dict
    """Generates dual authentication headers for a request."""

    nonce = os.urandom(NONCE_SIZE).hex()
    timestamp = iso_timestamp()

    payload = bytes(f"{api_key}{bearer_token}{timestamp}{nonce}", "utf-8")
    key = bytes.fromhex(signing_key)

    signature = hmac.new(key, payload, digestmod=hashlib.sha256).digest().hex()

    auth = f"Dual {bearer_token}:{api_key}:{signature}"

    return {
        "Nonce": nonce,
        "Timestamp": timestamp,
        "Authorization": auth,
    }



import base64
import os

from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import padding

from blockchyp.error import BlockChypError


IV_LENGTH = 16
BLOCK_SIZE = 128


def encrypt(plaintext, key):
    """Encrypts the string payload to base64 encoded ciphertext."""
    iv = os.urandom(IV_LENGTH)

    encryptor = Cipher(
        algorithms.AES(key),
        modes.CBC(iv),
        backend=default_backend(),
    ).encryptor()

    padder = padding.PKCS7(BLOCK_SIZE).padder()

    payload = padder.update(bytes(plaintext, "utf-8")) + padder.finalize()

    ciphertext = encryptor.update(payload) + encryptor.finalize()

    return base64.b64encode(iv + ciphertext).decode("utf-8")


def decrypt(ciphertext, key):
    """Decrypts base64 encoded ciphertext to a plaintext string."""
    raw = base64.b64decode(ciphertext)

    if len(raw) < IV_LENGTH:
        raise BlockChypError("Payload too short.")

    iv = raw[:IV_LENGTH]
    payload = raw[IV_LENGTH:]

    decryptor = Cipher(
        algorithms.AES(key),
        modes.CBC(iv),
        backend=default_backend(),
    ).decryptor()

    unpadder = padding.PKCS7(BLOCK_SIZE).unpadder()

    plaintext = decryptor.update(payload) + decryptor.finalize()

    unpadded = unpadder.update(plaintext) + unpadder.finalize()

    return unpadded.decode("utf-8")



"""
This library allows you to interact with the BlockChyp Terminal and
Gateway APIs.

For more information on this library, see the README.
    https://github.com/blockchyp/blockchyp-python
For more information on the Gateway API, see the docs:
    https://docs.blockchyp.com
"""


from blockchyp import util
from blockchyp import version
from blockchyp.types import CardType, PromptType, SignatureFormat
from blockchyp.blockchyp import Client


__version__ = version.VERSION
